
// C++17 fold expressions

#include <iostream>
#include <vector>
#include <climits>
#include <cstdint>
#include <type_traits>
#include <utility>

// print
template<typename ...Args>
void printer(Args&& ... args) {
    // we need parentheses around fold expression
    (std::cout << ... << args) << '\n';
}

// push_back more values
template<typename T, typename... Args>
void push_back_vec(std::vector<T>& v, Args&&... args)
{
    // check if all args are usable
    static_assert((std::is_constructible_v<T, Args&&> && ...));
    (v.push_back(std::forward<Args>(args)), ...);
}

// compile-time endianness swap based on http://stackoverflow.com/a/36937049 
template<class T, std::size_t... N>
constexpr T bswap_impl(T i, std::index_sequence<N...>) {
  return (((i >> N*CHAR_BIT & std::uint8_t(-1)) << (sizeof(T)-1-N)*CHAR_BIT) | ...);
}
template<class T, class U = std::make_unsigned_t<T>>
constexpr U bswap(T i) {
  return bswap_impl<U>(i, std::make_index_sequence<sizeof(T)>{});
}

int main()
{
    printer(1, 2, 3, " abc");

    std::vector<int> v;
    push_back_vec(v, 1, 2, 3, 4);
    push_back_vec(v, 5, 6, 7);
    for (int i : v) std::cout << i << ' ';
    std::cout << "\n";

    // check if bswap is correct and is compile-time expression
    static_assert(bswap<std::uint16_t>(0x1234u)==0x3412u);
    static_assert(bswap<std::uint64_t>(0x0123456789abcdefULL)==0xefcdab8967452301ULL);
}

