
// http://en.cppreference.com/w/cpp/utility/pair/tuple_size
// http://en.cppreference.com/w/cpp/utility/tuple/tuple_cat
// ...
// C++14


#include <tuple>
#include <iostream>
#include <utility>
#include <string>
#include <array>

/////////////////////////////////////////////////////////////////////////////
// helper function to print a tuple of any size
template<class Tuple, std::size_t N>
struct TuplePrinter {
    static void print(const Tuple& t)
    {
        TuplePrinter<Tuple, N-1>::print(t);
        std::cout << ", " << std::get<N-1>(t);
    }
};

template<class Tuple>
struct TuplePrinter<Tuple, 1> {
    static void print(const Tuple& t)
    {
        std::cout << std::get<0>(t);
    }
};

template<class... Args>
void print(const std::tuple<Args...>& t)
{
    std::cout << "(";
    TuplePrinter<decltype(t), sizeof...(Args)>::print(t);
    std::cout << ")\n";
}
// end helper function
/////////////////////////////////////////////////////////////////////////////

template<class T>
void test(T t)
{
    int a[std::tuple_size<T>::value]; // can be used at compile time ...
    std::cout << "size=" << std::tuple_size<T>::value << "\n"; // ... or at run time
    a[0]++;
}

#if (__cplusplus >= 201402L)
template <class... Args>
struct type_list
{
    template <std::size_t N>
        using type = typename std::tuple_element<N, std::tuple<Args...>>::type;
};
#endif

int main()
{
    std::array<float, 3> arr;
    test(arr);
    test(std::make_tuple(1, 2, 3.14));
    test(std::make_pair(1, 3.14));

    auto t = std::make_tuple(1, "Foo", 3.14);

    // index-based access
    std::cout << "index-based access in C++11: ";
    std::cout << "(" << std::get<0>(t) << ", " << std::get<1>(t)
              << ", " << std::get<2>(t) << ")\n";
    test(t);
    print(t);

#if (__cplusplus >= 201402L)
    // type-based access in C++14
    std::cout << "type-based access in C++14: ";
    std::cout << "(" << std::get<int>(t) << ", " << std::get<const char*>(t)
        << ", " << std::get<double>(t) << ")\n";
#endif

#if (__cplusplus >= 201402L)
    std::cout << std::boolalpha;
    type_list<int, char, bool>::type<2> x = true;
    std::cout << x << '\n';
#endif

    arr[0] = 1.0f;
    // concatenation of tuples
    auto t2 = std::tuple_cat(t, std::make_pair("Foo", "bar"), t, std::tie(arr[0]));
    arr[0] = 5.0f;
    print(t2);
    test(t2);
}

